@@ -3,8 +3,9 @@ use strict;
 use warnings;
 use Carp qw(croak);
 use Digest::SHA1;
-use Brackup::Util qw(tempfile slurp valid_params);
 use POSIX qw(mkfifo);
+use Brackup::DecryptedFile;
+use Brackup::Decrypt;
 
 sub new {
     my ($class, %opts) = @_;
@@ -12,7 +13,7 @@ sub new {
 
     $self->{to}      = delete $opts{to};      # directory we're restoring to
     $self->{prefix}  = delete $opts{prefix};  # directory/file filename prefix, or "" for all
-    $self->{file}    = delete $opts{file};    # filename we're restoring from
+    $self->{filename}= delete $opts{file};    # filename we're restoring from
     $self->{verbose} = delete $opts{verbose};
 
     $self->{prefix} =~ s/\/$// if $self->{prefix};
@@ -20,10 +21,9 @@ sub new {
     $self->{_stats_to_run} = [];  # stack (push/pop) of subrefs to reset stat info on
 
     die "Destination directory doesn't exist" unless $self->{to} && -d $self->{to};
-    die "Backup file doesn't exist"           unless $self->{file} && -f $self->{file};
     croak("Unknown options: " . join(', ', keys %opts)) if %opts;
 
-    $self->decrypt_file_if_needed;
+    $self->{metafile} = Brackup::DecryptedFile->new(filename => $self->{filename});
 
     return $self;
 }
@@ -39,20 +39,6 @@ sub _driver_meta {
     return $ret;
 }
 
-sub decrypt_file_if_needed {
-    my $self = shift;
-    my $meta = slurp($self->{file});
-    if ($meta =~ /[\x00-\x08]/) { # silly is-binary heuristic
-        my $new_file = $self->_decrypt_data(\$meta,
-                                            want_tempfile => 1,
-                                            no_batch => 1,
-                                            );
-        warn "Decrypted metafile $self->{file} to $new_file; using that to restore from...\n";
-        $self->{file} = $new_file;
-        scalar <STDIN>;
-    }
-}
-
 sub restore {
     my ($self) = @_;
     my $parser = $self->parser;
@@ -114,64 +100,6 @@ sub restore {
     }
 }
 
-sub _output_temp_filename {
-    my $self = shift;
-    return $self->{_output_temp_filename} ||= ( (tempfile())[1] || die );
-}
-
-sub _encrypted_temp_filename {
-    my $self = shift;
-    return $self->{_encrypted_temp_filename} ||= ( (tempfile())[1] || die );
-}
-
-sub _decrypt_data {
-    my $self = shift;
-    my $dataref = shift;
-    my %opts = valid_params(['no_batch', 'want_tempfile'], @_);
-
-    # find which key we're decrypting it using, else return chunk
-    # unmodified in the not-encrypted case
-    if ($self->{_meta}) {
-        my $rcpt = $self->{_meta}{"GPG-Recipient"} or
-            return $dataref;
-    }
-
-    unless ($ENV{'GPG_AGENT_INFO'} ||
-            @Brackup::GPG_ARGS ||
-            $self->{warned_about_gpg_agent}++)
-    {
-        my $err = q{#
-                        # WARNING: trying to restore encrypted files,
-                        # but $ENV{'GPG_AGENT_INFO'} not present.
-                        # Are you running gpg-agent?
-                        #
-                    };
-        $err =~ s/^\s+//gm;
-        warn $err;
-    }
-
-    my $output_temp = $opts{want_tempfile} ?
-        (tempfile())[1]
-        : $self->_output_temp_filename;
-    my $enc_temp    = $self->_encrypted_temp_filename;
-
-    _write_to_file($enc_temp, $dataref);
-
-    my @list = ("gpg", @Brackup::GPG_ARGS,
-                "--use-agent",
-                !$opts{no_batch} ? ("--batch") : (),
-                "--trust-model=always",
-                "--output",  $output_temp,
-                "--yes", "--quiet",
-                "--decrypt", $enc_temp);
-    system(@list)
-        and die "Failed to decrypt with gpg: $!\n";
-
-    return $output_temp if $opts{want_tempfile};
-    my $data = Brackup::Util::slurp($output_temp);
-    return \$data;
-}
-
 sub _update_statinfo {
     my ($self, $full, $it) = @_;
 
@@ -275,7 +203,7 @@ sub _restore_file {
             }
         }
 
-        my $decrypted_ref = $self->_decrypt_data($dataref);
+        my $decrypted_ref = Brackup::Decrypt::decrypt_data($dataref, meta => $self->{_meta});
         print $fh $$decrypted_ref;
     }
     close($fh) or die "Close failed";
@@ -303,22 +231,7 @@ sub _restore_file {
 # returns iterator subref which returns hashrefs or undef on EOF
 sub parser {
     my $self = shift;
-    return Brackup::Metafile->open($self->{file});
-}
-
-sub _write_to_file {
-    my ($file, $ref) = @_;
-    open (my $fh, ">$file") or die "Failed to open $file for writing: $!\n";
-    print $fh $$ref;
-    close($fh) or die;
-    die "Restored file is not of the correct size" unless -s $file == length $$ref;
-    return 1;
-}
-
-sub DESTROY {
-    my $self = shift;
-    unlink(grep { $_ } ($self->{_output_temp_filename},
-                        $self->{_encrypted_temp_filename}));
+    return Brackup::Metafile->open($self->{metafile}->name);
 }
 
 1;
